# -*- coding: utf-8 -*-

from .config import permissions, validate_ip
from .exceptions import (
    CyberarkAPIException, CyberarkException, UserAlreadyMember, UserNotExists, SafeAlreadyExistsError, BastionException
)
import json
import re
from .abstract import Bastion


class PrivilegedAccount:
    """Base class to be used with accounts fonctions"""
    def __init__(self, name: str, platformId: str, safeName: str,
                 platformAccountProperties: dict, secret: str = "", secretType:str = "Key",
                 secretManagement: dict = None,
                 remoteMachinesAccess: dict = None,
                 id: str = "",  address: str = "", userName: str = "",
                 **other):
        self.secret = secret
        if remoteMachinesAccess is None:
            remoteMachinesAccess = {"remoteMachines": "string", "accessRestrictedToRemoteMachines": True}
        if secretManagement is None:
            secretManagement = {"automaticManagementEnabled": True, "manualManagementReason": ""}
        self.remoteMachinesAccess = remoteMachinesAccess
        self.secretManagement = secretManagement
        self.secretType = secretType
        self.platformAccountProperties = platformAccountProperties
        self.safeName = safeName
        self.platformId = platformId
        self.userName = userName
        self.address = address
        self.name = name
        self.id = id
         
    def get_name(self):
        return "{}-{}".format(self.address, self.userName)

    def to_json(self):
        json_object = {"id": self.id, "name": self.get_name(), "address": self.address, "userName": self.userName,
                       "platformId": self.platformId, "safeName": self.safeName, "secret": self.secret,
                       "platformAccountProperties": self.platformAccountProperties,
                       "secretManagement": self.secretManagement}

        return json_object

    def __str__(self):
        strrepr = self.to_json()
        strrepr["secret"] = "***********"
        return str(strrepr)


class Account:
    def __init__(self, epv: Bastion):
        self.epv = epv

    def add_account_to_safe(self, account: PrivilegedAccount):
        url, head = self.epv.get_url('API/Accounts')

        try:
            req = self.epv.session.post(url, headers=head, data=json.dumps(account.to_json()), **self.epv.request_params)
        except Exception as err:
            # uncatched exception
            raise CyberarkException(err)

        if req.status_code != 201:
            content = json.loads(req.content)
            raise CyberarkAPIException(content["ErrorMessage"])

        acc_id = json.loads(req.content)['id']
        account.id = acc_id


    def is_valid_username(name: str) -> bool:
        return re.match("[^@]+@[^@]+[.][^@]+", name) is not None


    def is_valid_safename(name: str) -> bool:
        return re.match("^[0-9]{12}$", name) is not None


    def user_exists(self, username: str):
        if self.epv.user_list is None:
            url, head = self.epv.get_url('API/Users')

            req = self.epv.session.get(url, headers=head, **self.epv.request_params)
            if req.status_code != 200:
                raise Exception("Error occured during user_exists call. CyberArk error:", req.content)
            self.epv.user_list = [u['username'].lower().strip() for u in json.loads(req.content)['Users']]
        return username in self.epv.user_list




    def search_account_by_ip_addr(self, account: (PrivilegedAccount, str)) -> list:
        """
        This function search accounts using IPv4 address (or with address attribute of PrivilegedAccount object)
        :param account: A PrivilegedAccount object or IPv4 valid address
        :return: A list of "PrivilegedAccount" objects
        """
        if type(account) is str:
            address = account
        elif isinstance(account, PrivilegedAccount):
            address = account.address
        else:
            raise TypeError("search account function returned : First argument of account is not valid")

        if not validate_ip(address):
            raise TypeError("The address of the object was not considered as valid IPv4 address %s", address)

        url, head = self.epv.get_url('API/Accounts?search=' + address)

        if isinstance(account, PrivilegedAccount):
            # Searching for both username + IP Address
            url += " " + account.userName + "&limit=111"
        else:
            url += "&limit=111"

        req = self.epv.session.get(url=url, headers=head, **self.epv.request_params)

        if req.status_code != 200:
            raise Exception("Error occured during search_account call. CyberArk error:")

        json_response_data = json.loads(req.content)
        exact_matches = []
        for result in json_response_data['value']:
            if result['platformAccountProperties']['IP'] == address:
                exact_matches.append(Account(result))
        return exact_matches

    def search_account(self, expression: str) -> dict:
        """
        This function search an account using free text search and return a list a Privileged Account objects
        :param expression: List of keywords to search for in accounts, separated by a space.
        :return: List of PrivilegedAccount objects
        """
        url, head = self.epv.get_url("API/Accounts?search=" + expression)
        req = self.epv.session.get(url=url, headers=head, **self.epv.request_params)
        if req.status_code != 200:
            raise CyberarkException("An error occured during search_account call.")
        acc_list = {}
        for acc in json.loads(req.content)['value']:
            acc_list[acc["id"]] = PrivilegedAccount(**acc)
        return acc_list

    def search_account_by(self, keywords=None, username=None, address=None, safe=None,
                          platform=None, name=None):
        """
        This function allow to search using one or more parameters and return list of account id
        :param keywords: free search
        :param username: username search (field "userName")
        :param address: IP adresse search (field ['platformAccountProperties']['IP'])
        :param safe: search in particular safe
        :param platform: search by platform name (no space) (field "platformId")
        :param name: search by account name (field "name")
        :return: a list of account id
        """
        if keywords is None and username is None and address is None and safe is None and platform is None \
                and name is None:
            raise BastionException("Please provide at least one search topic")

        search = "search="
        add_space = ""
        if keywords:
            search += keywords
            add_space = " "
        if username:
            search += add_space + username
            add_space = " "
        if address:
            search += add_space + address
            add_space = " "
        if platform:
            search += add_space + platform
            add_space = " "
        if name:
            search += add_space + name
        if safe:
            if search != "search=":
                search += "&filter= safeName eq " + safe
            else:
                search = "filter=safeName eq " + safe
        search += "&limit=1000"

        url, head = self.epv.get_url("API/Accounts?" + search)
        req = self.epv.session.get(url=url, headers=head, **self.epv.request_params)

        if req.status_code != 200:
            raise Exception("Error occured during search_account call. CyberArk error:")

        json_response_data = json.loads(req.content)
        exact_matches = []
        if address:
            for result in json_response_data['value']:
                if 'IP' in result['platformAccountProperties']:
                    if result['platformAccountProperties']['IP'] == address:
                        exact_matches.append(result['id'])
        if username:
            for result in json_response_data['value']:
                if result['userName'] == username:
                    exact_matches.append(result['id'])
        if platform:
            for result in json_response_data["value"]:
                if result['platformId'] == platform:
                    exact_matches.append(result['id'])
        if name:
            for result in json_response_data["value"]:
                if result['name'] == platform:
                    exact_matches.append(result['id'])
        return exact_matches
    
    def connect_using_PSM(self, accountId, connectionComponent):
        url, head = self.epv.get_url('API/Accounts/' + accountId + '/PSMConnect')
        head["Accept"] = 'RDP'
        
        body = {"ConnectionComponent": connectionComponent}
        try:
            req = self.epv.session.post(url, headers=head, data=json.dumps(body), **self.epv.request_params)
        except Exception as err:
            # uncatched exception
            raise CyberarkException(err)

        if req.status_code != 200:
            content = json.loads(req.content)
            raise CyberarkAPIException(content["ErrorMessage"])

        content = str(req.content)
        return req.content