from .abstract import Bastion
from .exceptions import *
import json


class Applications:
    def __init__(self, epv: Bastion):
        self.epv = epv

    def handle(self, url, head, body):
        try:
            req = self.epv.session.post(url, headers=head, data=json.dumps(body), **self.epv.request_params)
        except Exception as err:
            raise BastionException(err)

        if req.status_code != 201:
            raise CyberarkException(req.content)
            # content = json.loads(req.content)
            # raise CyberarkAPIException(content["ErrorMessage"])

        return True

    def get_application(self, app_name: str):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Applications/' + app_name)
        req = self.epv.session.get(url, headers=head, **self.epv.request_params)

        if req.status_code == 200:
            return json.loads(req.content)
        elif req.status_code == 404:
            return False
        else:
            raise Exception("Error occured during get_members call. CyberArk error:", req.content)

    def add_authentication(self, app_name: str, path: str = None, hash_string: str = None, os_user: str = None,
                           address: str = None, serial_number: str = None, issuer: list = None, subject: list = None,
                           subject_alternative_name: list = None, is_folder: bool = False,
                           allow_internal_scripts: bool = False, comment: str = "") -> bool:
        """
        Add authentication method to a given app_id with a named param
        :param app_name: the name of the application
        :param path: path to authenticated
        :param hash_string: hash of script / binary
        :param os_user: os user that is running the script / binary
        :param address: IP address
        :param serial_number: certificate serial number
        :param issuer: list of certificate issuer (PVWA >= 11.4)
        :param subject: list of certificate subject (PVWA >= 11.4)
        :param subject_alternative_name: list of certificate SAN (eg ["DNS Name=www.example.com","IP Address=1.2.3.4"])
        :param allow_internal_scripts: relevant for path authentication only (False by default)
        :param is_folder: relevant for path authentication only (False by default)
        :param comment: relevant for hash and certificate serial number
        :return: boolean telling wether the application was updated or not
        """

        updated = False

        url, head = self.epv.get_url('WebServices/PIMServices.svc/Applications/' + app_name + '/Authentications/')

        if path is not None:
            body = {
                "authentication": {
                    "AuthType": "path",
                    "AuthValue": path,
                    "IsFolder": is_folder,
                    "AllowInternalScripts": allow_internal_scripts
                }
            }
            updated = self.handle(url, head, body)

        if hash_string is not None:
            body = {
                "authentication": {
                    "AuthType": "hash",
                    "AuthValue": hash_string,
                    "Comment": comment
                }
            }
            updated = self.handle(url, head, body)

        if os_user is not None:
            body = {
                "authentication": {
                    "AuthType": "osUser",
                    "AuthValue": os_user
                }
            }
            updated = self.handle(url, head, body)

        if address is not None:
            body = {
                "authentication": {
                    "AuthType": "machineAddress",
                    "AuthValue": address
                }
            }
            updated = self.handle(url, head, body)

        if serial_number is not None:
            body = {
                "authentication": {
                    "AuthType": "certificateserialnumber",
                    "AuthValue": serial_number,
                    "Comment": comment
                }
            }
            updated = self.handle(url, head, body)

        if issuer is not None or subject is not None or subject_alternative_name is not None:
            if isinstance(issuer, str):
                issuer = [issuer]
            if isinstance(subject, str):
                subject = [subject]
            if isinstance(subject_alternative_name, str):
                subject_alternative_name = [subject_alternative_name]

            body = {
                "authentication": {
                    "AuthType": "certificateattr",
                }
            }

            if issuer:
                body["authentication"]["Issuer"] = issuer
            if subject:
                body["authentication"]["Subject"] = subject
            if subject_alternative_name:
                body["authentication"]["SubjectAlternativeName"] = subject_alternative_name

            updated = self.handle(url, head, body)

        return updated
