# -*- coding: utf-8 -*-

import requests
import json
from .exceptions import CyberarkException, GetTokenException, BastionException
from .config import Config
from .accounts import Account
from .platforms import Platform
from .safe import Safe
from .abstract import Bastion
from .users import User, Group
from .applications import Applications


class EPV(Bastion):
    def __init__(self, configfile: str = None, serialized: dict = None):
        if configfile is None and serialized is None:
            raise BastionException("You must provide either configfile or serialized to init EPV")
        if configfile is not None and serialized is not None:
            raise BastionException("You must provide either configfile or serialized to init EPV, not both")
        if configfile is not None:
            self.config = Config(configfile)
            self.api_host = self.config.PVWA
            self.request_params = {"timeout": self.config.timeout, "verify": self.config.CA}
            self.api_host = self.config.PVWA
            self.cpm = self.config.CPM
            self.retention = self.config.retention
            self.__token = None

        if serialized is not None:
            self.request_params = {"timeout": serialized["timeout"], "verify": serialized["verify"]}
            self.api_host = serialized['api_host']
            self.cpm = serialized['cpm']
            self.retention = serialized['retention']
            self.__token = serialized['token']

        self.session = requests.Session()
        self.user_list = None

        # utilities
        self.account = Account(self)
        self.platform = Platform(self)
        self.safe = Safe(self)
        self.user = User(self)
        self.group = Group(self)
        self.application = Applications(self)

    # start of functions definition
    def __login_cyberark(self, username: str, password: str, auth_type: str) -> str:
        assert self.__token is None
        assert auth_type.upper() in ("CYBERARK", "WINDOWS", "LDAP", "RADIUS")
        url, head = self.get_url("API/Auth/" + auth_type + "/Logon")
        request_data = {"username": username, "password": password}
        try:
            req = self.session.post(url, headers=head, data=json.dumps(request_data), **self.request_params)
            # Cleaning password after authentication
            self.__password = ""
        except ConnectionError or TimeoutError or requests.exceptions.ConnectTimeout:
            raise CyberarkException("Network problem connecting to PVWA")
        if req.status_code != 200:
            if req.status_code == 403:
                raise CyberarkException("Invalid credentials ! " + str(req.status_code))
            else:
                raise CyberarkException("Unknown error")
        return json.loads(req.content)

    def logoff(self):
        url, head = self.get_url("API/Auth/CyberArk/Logoff")
        req = self.session.post(url, headers=head, **self.request_params)
        if req.status_code != 201:
            raise CyberarkException("Error disconnecting to PVWA")
            pass
        self.__token = None

    def check_token(self) -> bool or None:
        if self.__token is None:
            return None
        url, head = self.get_url("WebServices/PIMServices.svc/User")
        req = self.session.get(url, headers=head, **self.request_params)
        if req.status_code != 200:
            self.__token = None
            return False
        return True

    def login(self, username=None, password=None, auth_type="Cyberark"):
        if self.check_token():
            return
        try:
            # TODO add AIM call to get user / password
            # account = self.get_password_object(self.__object)
            # user = account['UserName']
            # password = account['Content']
            pass
        except PermissionError:
            raise GetTokenException("I was unable to grab password")
        except OSError:
            raise GetTokenException("i was unable to read the tmp file ")

        if username is None:
            if self.config.username is None:
                raise BastionException("Username must be provided on login call or in configuration file")
            username = self.config.username
        if password is None:
            if self.config.password is None:
                raise BastionException("Password must be provided on login call or in configuration file")
            password = self.config.password

        if self.config.authtype is not None:
            auth_type = self.config.authtype

        try:
            self.__token = self.__login_cyberark(username, password, auth_type)
        except CyberarkException as err:
            raise GetTokenException(err)

    def get_url(self, url) -> (str, str):
        addr = 'https://' + self.api_host + '/PasswordVault/' + url
        head = {"Content-type": "application/json", "Authorization": self.__token}
        return addr, head

    def to_json(self):
        serialized = {
            "api_host": self.config.PVWA,
            "timeout": self.config.timeout,
            "verify": self.config.CA,
            "cpm": self.config.CPM,
            "retention": self.config.retention,
            "token": self.__token,
        }
        return serialized
