# -*- coding: utf-8 -*-

from .config import permissions, validate_ip
from .exceptions import (
    CyberarkAPIException, CyberarkException, UserAlreadyMember, UserNotExists, SafeAlreadyExistsError, BastionException
)
import json
import re
from .abstract import Bastion


class Safe:
    def __init__(self, epv: Bastion):
        self.epv = epv

    def add_safe_member(self, safe: str, username: str, profile: str):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Safes/' + safe + '/Members')

        data = {'member': {
            'MemberName': username,
            'Permissions': permissions(profile)
        }
        }
        req = self.epv.session.post(url, headers=head, data=json.dumps(data), **self.epv.request_params)
        if req.status_code != 201:
            content = json.loads(req.content)
            if "Reason: User or Group is already a member." in content["ErrorMessage"]:
                raise UserAlreadyMember
            elif "Reason: Input parameter for [MemberName] value is invalid" in content["ErrorMessage"]:
                raise UserNotExists
            else:
                raise Exception("Error occured during add_safe_member call. CyberArk error:", req.content)

    def remove_safe_member(self, safe: str, username: str):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Safes/' + safe + '/Members/' + username)
        req = self.epv.session.delete(url, headers=head, **self.epv.request_params)

        if req.status_code != 200:
            raise Exception("Error occured during remove_safe_member call. CyberArk error:", req.content)

    def safe_exists(self, safename: str):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Safes/?query=' + safename)
        req = self.epv.session.get(url, headers=head, **self.epv.request_params)
        if req.status_code == 200:
            return True
        elif req.status_code == 404:
            return False
        else:
            raise Exception("Error occured during remove_safe_member call. CyberArk error:", req.content)

    def create_safe(self, safename: str, description=""):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Safes')
        data = {
            "safe": {
                "SafeName": safename,
                "Description": description,
                "OLACEnabled": False,
                "ManagingCPM": self.epv.cpm,
                "NumberOfVersionsRetention": self.epv.retention
            }
        }

        try:
            req = self.epv.session.post(url=url, data=json.dumps(data), headers=head, **self.epv.request_params)
        except Exception:
            raise CyberarkException

        if req.status_code != 201:
            error_code = req.json()["ErrorCode"]
            if error_code == 'CAWS00001E':
                raise SafeAlreadyExistsError()
            else:
                raise CyberarkException("Error occured during add_safe call. CyberArk error:")
        return 0

    def add_safe(self, safename: str, description=""):
        try:
            self.create_safe(safename, description)
        except Exception as e:
            raise e

        # Define Safe defaults owners
        for user, profile in {"Vault Admins": "admin", "onboarding": "prov", "Administrator": 'admin'}.items():
            try:
                self.add_safe_member(safename, user, profile)
            except UserAlreadyMember:
                pass
        # when we did that, we had a bug so we remove it
        # TODO : try to remove admins right to provisionning ?
        # try:
        #     # Removing provisioning user from admin of the safe
        #     self.remove_safe_member(safename, config.username)
        # except CyberarkException:
        #     raise CyberarkException("Unable to remove %s from safe", config.username)

    def get_members(self, safe_name: str):
        url, head = self.epv.get_url('WebServices/PIMServices.svc/Safes/' + safe_name + '/Members')
        req = self.epv.session.get(url, headers=head, **self.epv.request_params)

        if req.status_code == 200:
            return json.loads(req.content)
        elif req.status_code == 404:
            return False
        else:
            raise Exception("Error occured during get_members call. CyberArk error:", req.content)

